/* ftpdctl - control Moylan ftp server
   Commands are start stop status etc.
   Return 0 if OK else error code

   Copyright (c) 2009-2021 Steven Levine and Associates, Inc.
   All rights reserved.

   2009-11-12 SHL Baseline
   2017-08-09 SHL Support d:\logs
   2017-08-11 SHL Switch to FindLogFiles
   2017-08-11 SHL Sync with templates
   2017-08-13 SHL Convert to Globals
   2017-09-18 SHL Add monitor daemon
   2018-12-11 SHL Use SetFocus
   2019-03-21 SHL Sync with renames
   2019-04-05 SHL Sync with templates - RunKiller etc.
   2020-04-24 SHL Sync with templates - SetFocus
   2020-10-15 SHL Sync with .hlp file move
   2021-03-15 SHL Sync with templates
   2021-03-15 SHL Support detached keyword
   2021-03-16 SHL Start monitor in separate session
   2021-04-16 SHL Sync with templates
   2021-04-16 SHL Drop -f support - use startf
*/

/* use 'setlocal' until we know how/when call setlocal corrupts environment */
'@setlocal 2>nul'
envSaved = RC = 0

signal on Error
signal on Failure name Error
signal on Halt
signal on NotReady name Error
signal on NoValue name Error
signal on Syntax name Error

gVersion = '0.1 2021-04-16'

Globals = 'gAppDir gAppName gAppTitle gArgList. gCfgExe gCfgFile gCmdName',
	  'gEditor gEnv gErr gErrCondition gHelpViewer gHostName',
	  'gKiller gMonitorExe gPid gRxuLoaded gTMPDIR gTmpDir gVerbose gVersion'

call Initialize

gAppTitle = 'Moylan FTP deamon'
gAppName = 'ftpd'
gMonitorExe = 'monitor'
gCfgExe = 'setup'
gCfgFile = 'ftpd.ini'

call FindAppDir

/* Run from application directory */
if \ envSaved then do
  oldCwd = directory()
  oldCwd2 = directory(left(gAppDir, 2))
end
call directory gAppDir

if \ IsExeInPath(gAppName) then
  call Die gAppTitle 'not found in PATH'

if \ IsExeInPath(gCfgExe) then
  call Die gCfgExe 'not found in PATH'

if \ IsExeInPath(gCfgFile) then
  call Die gCfgFile 'not found in' gAppDir

Main:
  parse arg cmdLine
  call ScanArgs cmdLine
  drop cmdLine

  gErr = 0
  gRxuLoaded = 0

  do argNum = 1 to gArgList.0
    curArg = gArgList.argNum
    call DoArg curArg
  end

  if \ envSaved then do
    if oldCwd \== oldCwd2 then
      call directory oldCwd2
    call directory oldCwd
  end

  exit gErr

/* end main */

/*=== DoArg(action) Process requests, return rc ===*/

DoArg: procedure expose (Globals)
  parse arg curArg
  select
  when curArg == 'status' then
    call DoStatus
  when curArg == 'start' then
    call DoStart
  when curArg == 'startf' then
    call DoStartf
  when curArg == 'detach' then
    call DoDetach
  when curArg == 'monitor' then
    call DoMonitor
  when curArg == 'stop' then
    call DoStop
  when curArg == 'log' then
    call DoLog
  when curArg == 'config' then
    call DoConfig
  when curArg == 'kill' then
    call DoKill
  when curArg == 'restart' then
    call DoRestart
  when curArg == 'help' then
    call DoViewHelp
  otherwise
    call ScanArgsUsage 'Request' curArg 'unexpected'
  end
  return

/* end DoArg */

/*=== DoConfig() Config ===*/

DoConfig: procedure expose (Globals)
  'start' gCfgExe 'L'
  say gAppTitle 'configurator started'
  return

/* end DoConfig */

/*=== DoDetach() Start server detached ===*/

DoDetach: procedure expose (Globals)
  if IsProcessRunningEx() then
    say gAppTitle 'already running and can be accessed remotely as' gHostName
  else do
    if gVerbose then
      say 'Starting' gAppTitle 'detached from' directory()
    signal off Error
    'detach' gAppName
    signal on Error
    if RC \= 0 & RC \= 457 then
      signal Error
    if IsProcessRunningEx(1) then
      say gAppTitle 'started and can be accessed remotely as' gHostName
    else do
      say 'Can not start' gAppTitle
      gErr = 1
    end
  end
  return

/* end DoDetach */

/*=== DoKill() Kill ===*/

DoKill: procedure expose (Globals)
  if \ IsProcessRunningEx() then
    say gAppTitle 'is not running'
  else do
    call RunKiller '-KILL'
    if IsProcessRunningEx(0) then do
      say gAppTitle 'will not die'
      gErr = 1
    end
    else
      say gAppTitle 'has stopped'
  end
  return

/* end DoKill */

/*=== DoLog() Log ===*/

DoLog: procedure expose (Globals)

  files = FindLogFiles()

  if files == '' then
    say 'Cannot find any ftp logs files - maybe not yet created'
  else do
    call FindEditor
    say 'Starting' gAppTitle 'viewer'
    gEditor files
  end
  return

/* end DoLog */

/*=== DoMonitor() Start monitor ===*/

DoMonitor: procedure expose (Globals)
  appNameSave = gAppName
  gAppName = gMonitorExe
  appTitle = 'Monitor'
  if IsProcessRunningEx() then
    say appTitle 'already running'
  else do
    say 'Starting' appTitle 'from' directory()
    signal off Error
    'start' gAppName
    signal on Error
    if RC \= 0 & RC \= 457 then do
      say appTitle 'start failed with rc ' RC
      gErr = 1
    end
  end
  gAppName = appNameSave
  return

/* end DoMonitor */

/*=== DoRestart() Restart ===*/

DoRestart: procedure expose (Globals)
  call DoStop
  if gErr = 0 then
    call DoStart
  else
    say gAppTitle 'stop failed'
  return

/* end DoRestart */

/*=== DoStart() Start server minimized ===*/

DoStart: procedure expose (Globals)
  if IsProcessRunningEx() then
    say gAppTitle 'already running and can be accessed remotely as' gHostName
  else do
    if gVerbose then
      say 'Starting' gAppTitle 'from' directory()
    signal off Error
    'start "' || gAppTitle || '" /min /c' gAppName
    signal on Error
    if RC \= 0 & RC \= 457 then
      signal Error
    if IsProcessRunningEx(1) then
      say gAppTitle 'started and can be accessed remotely as' gHostName
    else do
      say 'Can not start' gAppTitle
      gErr = 1
    end
    call SetFocus
  end
  return

/* end DoStart */

/*=== DoStartf() Start server in foreground ===*/

DoStartf: procedure expose (Globals)
  if IsProcessRunningEx() then
    say gAppTitle 'already running and can be accessed remotely as' gHostName
  else do
    if gVerbose then
      say 'Starting' gAppTitle 'from' directory()
    signal off Error
    gAppName
    signal on Error
    if RC \= 0 then do
      say gAppTitle 'exited with rc ' RC
      gErr = 1
    end
  end
  return

/* end DoStartf */

/*=== DoStatus() Status ===*/

DoStatus: procedure expose (Globals)
  if IsProcessRunningEx() then do
    if \ gVerbose then do
      say gAppTitle 'is running as Pid' gPid '('d2x(gPid)') and'
      say 'can be accessed remotely as' gHostName
    end
  end
  else do
    say gAppTitle 'is not running'
    gErr = 1
  end
  return

/* end DoStatus */

/*=== DoStop() Stop ===*/

DoStop: procedure expose (Globals)
  if \ IsProcessRunningEx() then
    say gAppTitle 'is not running'
  else do
    if \ gRxuLoaded then do
      call LoadRxu
      gRxuLoaded = 1
    end
    semName = "\SEM32\FTPSERVER\SHUTDOWN"
    /* Requires LoadRxu */
    if RxOpenEventSem('hev', semName) \= 0 then
      say 'Can not open' gAppTitle 'shutdown semaphore'
    else do
      call RxPostEventSem hev
      call RxResetEventSem hev
      call RxCloseEventSem hev
    end
    if IsProcessRunningEx(0) then do
      say gAppTitle 'did not stop'
      gErr = 1
    end
    else do
      say gAppTitle 'has stopped'
      f1 = gTmpDir '\ftptrans.log'
      /* 2011-06-01 SHL fixme to be gone when no longer needed */
      f2 = gTmpDir '\ftptrans.$$$'
      if IsFile(f1) & IsFile(f2) then do
	say 'Appending' f2 'to' f1
	'copy' f1 || '+' || f2 f1
	call SysFileDelete f2
      end
    end
  end
  return

/* end DoStop */

/*=== DoViewHelp() View help ===*/

DoViewHelp: procedure expose (Globals)
  helpfile = gAppDir || '\doc\ftpserver.inf'
  if \ IsFile(helpfile) then
    say 'Cannot access' helpfile
  else do
    call FindHelpViewer
    /* Start avoids error 184 when using existing session */
    'start' gHelpViewer helpfile
  end
  return

/* end DoViewHelp */

/*=== FindAppDir() Find base directory and set gAppDir and gHostName or die ===*/

FindAppDir: procedure expose (Globals)

  if symbol('gAppDir') \== 'VAR' then do
    do 1
      s = GetEnv('HOSTNAME')
      /* Try current directory */
      d = directory()
      s = 'ftp.' || s || '.com'		/* Guess same as host name */
      if IsFile(gCfgFile) & IsFile(MakePath(gAppName, '.exe')) then leave
      /* Try slainc */
      d = 'd:\Internet\FtpServer'
      s = 'ftp.warpcave.com'
      if IsDir(d) then leave
      /* Try dnacih */
      d = 'd:\apps\ftp'
      if IsDir(d) then do
	select
	when s == 'hal' then
	  s 'data.dnacih.com'
	when s == 'mail2' then
	  s = 'mail2.dnacih.com'
	otherwise
	  s = GetEnv('HOSTNAME')
	  s = 'ftp.' || s || '.com'	/* Guess same as host name */
	end
	leave
      end
      call Die 'Can not guess' gAppTitle 'base directory for host' s
    end /* do */
    gAppDir = d
    gHostName = s
  end /* if */
  return

/* end FindAppDir */

/*=== FindLogFiles() Find log files and return list ===*/

FindLogFiles: procedure expose (Globals)

  /* Look in appdir\logs %LOGFILES, d:\logs or %TMP */
  dirs = ''
  dir = MakePath(gAppDir, 'logs')	/* Application default */
  if IsDir(dir) then
    dirs = dirs dir
  dir = value('LOGFILES',, gEnv)	/* Generic default */
  if dir \== '' then
    dirs = dirs dir
  dir = 'd:\logs'			/* Well known location */
  if IsDir(dir) then
    dirs = dirs dir
  if symbol('GTMPDIR') == 'VAR' then
    dirs = dirs gTmpDir			/* Last chance */

  wildCards = 'ftpusers.$$$ ftpcommon.log ftptrans.$$$ ftpusers.log ftpcommon.log ftptrans.log'

  files = ''
  do dirNum = 1 to words(dirs)
    do wcNum = 1 to words(wildCards)
      wc = MakePath(word(dirs, dirNum), word(wildCards, wcNum))
      call SysFileTree wc, 'files', 'O'
      if RESULT \= 0 then
	call Die 'FindLogFiles' wc 'failed'
      do i = 1 to files.0
	files = files files.i
      end
    end /* wcNum */
    if files \== '' then
      leave				/* Stop on 1st match */
  end /* dirNum */

  return strip(files)

/* end FindLogFiles */

/*=== FindHelpViewer() Find help viewer ===*/

FindHelpViewer: procedure expose (Globals)

  if symbol('gHelpViewer') \== 'VAR' then do
    do 1
      gHelpViewer = 'viewx'
      if IsExeInPath(gHelpViewer) then leave
      gHelpViewer = 'ibmview'
      if IsExeInPath(gHelpViewer) then leave
      gHelpViewer = 'view'
      if IsExeInPath(gHelpViewer) then leave
      gHelpViewer = 'newview'
      if IsExeInPath(gHelpViewer) then leave
      call Die 'Can not select help viewer'
    end
  end
  return

/* end FindHelpViewer */

/*=== Initialize() Initialize globals ===*/

Initialize: procedure expose (Globals)
  call SetCmdName
  call LoadRexxUtil
  call LoadRxUtilEx
  gEnv = 'OS2ENVIRONMENT'
  call SetTmpDir
  return

/* end Initialize */

/*=== ScanArgsInit() ScanArgs initialization exit routine ===*/

ScanArgsInit: procedure expose (Globals) cmdTail swCtl keepQuoted

  if cmdTail == '' then
    call ScanArgsHelp			/* Show script help if no args */

  /* Preset defaults */
  gVerbose = 0				/* Verbose messages */
  gArgList.0 = 0			/* Reset arg count */
  return

/* end ScanArgsInit */

/*=== ScanArgsSwitch() ScanArgs switch option exit routine ===*/

ScanArgsSwitch: procedure expose (Globals) curSw curSwArg
  select
  when curSw == 'f' then
    call ScanArgsUsage '-f is obsolete - use startf'
  when curSw == 'h' | curSw == '?' then
    call ScanArgsHelp
  when curSw == 'v' then
    gVerbose = 1
  when curSw == 'V' then do
    say gCmdName gVersion
    exit
  end
  otherwise
    call ScanArgsUsage 'switch '''curSw''' unexpected'
  end /* select */
  return

/* end ScanArgsSwitch */

/*=== ScanArgsArg() ScanArgs argument option exit routine ===*/

ScanArgsArg: procedure expose (Globals) curArg
  i = gArgList.0 + 1
  gArgList.i = curArg
  gArgList.0 = i
  return

/* end ScanArgsArg */

/*=== ScanArgsTerm() ScanArgs scan end exit routine ===*/

ScanArgsTerm: procedure expose (Globals)
  if gArgList.0 = 0 then
    call ScanArgsUsage 'action required (i.e. start, stop)'
  return

/* end ScanArgsTerm */

/*=== ScanArgsHelp() Display ScanArgs usage help exit routine ===*/

ScanArgsHelp:
  say
  say 'Control Moylan FTP server for' gHostName
  say
  say 'Usage:' gCmdName '[-h] [-v] [-V] [-?] action...'
  say
  say '  -h -?   Display this message'
  say '  -v      Enable verbose output'
  say '  -V      Display version'
  say
  say '  action  start startf detach stop status log config monitor restart kill help'
  say '            startf - start ftp server in minimized'
  say '            startf - start ftp server in foreground'
  say '            detach - start start server detached'
  say '            monitor - start ftp daemon monitor'
  exit 255

/* end ScanArgsHelp */

/*=== ScanArgsUsage(message) Report Scanargs usage error exit routine ===*/

ScanArgsUsage:
  parse arg msg
  say
  if msg \== '' then
    say msg
  say 'Usage:' gCmdName '[-h] [-v] [-V] [-?] action...'
  exit 255

/* end ScanArgsUsage */

/*==============================================================================*/
/*=== SkelRexxFunc standards - Delete unused - Move modified above this mark ===*/
/*==============================================================================*/

/*=== FindEditor() Find non-GUI editor and set gEditor or Die ===*/

FindEditor: procedure expose (Globals)
  /* Requires GetEnv */
  /* Requires IsExeInPath */
  do while symbol('gEditor') \== 'VAR'
    gEditor = GetEnv('EDITOR')
    if gEditor \== '' then leave
    gEditor = 'vim.exe'			/* Force ext in case have vim alias */
    if IsExeInPath(gEditor) then leave
    gEditor = '4os2 /c vimx'
    if IsExeInPath('vimx.cmd') then leave
    gEditor = 'tedit'
    if IsExeInPath(gEditor) then leave
    call Die 'EDITOR not defined and cannot guess editor to use'
  end /* while */
  return

/* end FindEditor */

/*=== GetEnv(var) Return value for environment variable or empty string ===*/

GetEnv: procedure expose (Globals)
  parse arg var
  if var = '' then
    call Die 'GetEnv requires an argument'
  return value(var,, gEnv)

/* end GetEnv */

/*=== GetPidForProcess(procName) Return decimal pid for named process or empty string ===*/

GetPidForProcess: procedure
  /* Requires LoadRxUtilEx */
  parse arg procName
  /* If process name omitted, get own pid */
  if procName \= '' then
    req = 'N'
  else do
    procName = 0
    req = 'P'
  end
  /* Get pid parent-pid process-type priority cpu-time executable-name */
  info = Sys2QueryProcess(procName, req)
  if info == '' then
    decpid = ''				/* Failed */
  else
    parse var info decpid .
  return decpid

/* end GetPidForProcess */

/*=== IsDir(dirName[, full]) return true if directory is valid, accessible directory ===*/

IsDir: procedure
  /* If arg(2) not omitted, return full directory name or empty string */
  parse arg dir, full
  newdir = ''

  do 1
    if dir == '' then do
      cwd = ''				/* No restore needed */
      leave
    end
    dir = translate(dir, '\', '/')	/* Convert to OS/2 slashes */
    s = strip(dir, 'T', '\')		/* Chop trailing slashes unless root */
    if s \== '' & right(s, 1) \== ":" then
      dir = s				/* Chop */
    drv = filespec('D', dir)
    cwd = directory()			/* Remember */
    /* If have drive letter and requested directory on some other drive */
    if drv \== '' & translate(drv) \== translate(left(cwd, 2)) then do
      /* Avoid slow failures and unwanted directory changes */
      drvs = SysDriveMap('A:')
      if pos(translate(drv), drvs) = 0 then
	leave				/* Unknown drive */
      if SysDriveInfo(drv) == '' then
	leave				/* Drive not ready */
      cwd2 = directory(drv)		/* Remember current directory on other drive */
      newdir = directory(dir)		/* Try to change and get full path name */
      call directory cwd2		/* Restore current directory on other drive */
      leave
    end

    /* If no drive letter or same drive and not UNC name */
    if left(dir, 2) \== '\\' then do
      newdir = directory(dir)		/* Try to change and get full path name */
      leave
    end

    /* UNC name - hopefully server is accessible or this will be slow
       Accept
	 \\server
	 \\server\
	 \\server\dir\
	 \\server\dir
     */
    cwd = ''				/* No restore needed */
    wc = dir
    if right(wc, 1) \== '\' then
      wc = wc || '\'
    i = lastpos('\', wc)
    if substr(wc, 3, 1) == '\' then
      leave				/* Malformed UNC - no server name */
    if pos('*', wc) > 0 | pos('?', wc) > 0 then
      leave				/* No wildcards allowed */
    call SysFileTree wc, 'files', 'O'
    if files.0 > 0 then do
      s = files.1			/* Exists and is not empty */
      i = lastpos('\', s)
      newdir = left(s, i - 1)		/* Extract directory name from full path name */
      leave
    end
    /* Try wildcarded directory name */
    wc = strip(wc, 'T', '\')
    i = lastpos('\', wc)
    base = substr(wc, i + 1)
    if base == '' then
      leave				/* Should have matched above */
    wc = substr(wc, 1, i) || '*' || base || '*'
    call SysFileTree wc, 'files', 'DO'
    do fileNum = 1 to files.0
      /* Find directory name is list */
      s = files.fileNum
      i = lastpos('\', s)
      s2 = substr(s, i + 1)
      if translate(base) == translate(s2) then do
	newdir = left(s, i - 1)
	leave
      end
    end /* i */
  end /* 1 */

  if cwd \== '' then
    call directory cwd			/* Restore original directory and drive */

  if full \== '' then
    ret = newdir			/* Return full directory name or empty string */
  else
    ret = newdir \== ''			/* Return true if valid and accessible */
  return ret

/* end IsDir */

/*=== IsExeInPath(exe, wantPath) return TRUE if executable is in PATH or full path ===*/

/**
 * @param exe is executable to find, .exe assumed if no extension specified
 * @param wantPath optionally requests full path return
 * @returns TRUE if executable found in PATH, FALSE if not
 * @returns full path or emtpy string if wantPath not empty
 */

IsExeInPath: procedure
  parse arg exe, wantPath
  if exe \ == '' then do
    i = lastpos('.', exe)
    j = lastpos('\', exe)
    if i = 0 | i < j then
      exe = exe || '.exe'		/* No extension */
    exe = SysSearchPath('PATH', exe)
  end
  if wantPath == '' then
    exe = exe \== ''			/* Want bool result */

  return exe				/* Return bool or full path */

/* end IsExeInPath */

/*=== IsFile(file) return true if arg is file and file exists ===*/

IsFile: procedure expose (Globals)
  parse arg file
  if file == '' then
    yes = 0
  else do
    /* '.' and '..' returns files in '.' or '..' - so avoid false positives */
    call SysFileTree file, 'files', 'F'
    if RESULT \= 0 then
      call Die 'SysFileTree' file 'failed'
    /* Assume caller knows if arg contains wildcards */
    yes = file \== '.' & file \== '..' & files.0 \= 0
  end
  return yes

/* end IsFile */

/*=== IsProcessRunningEx([waitFor]) Return TRUE if app running, optionally waits 10 seconds for app start ===*/

/**
 * Check if gAppName process is running and return 1 if running
 * Sets gPid if gAppName process is running
 * Optionally wait up to 10 seconds for process to start
 * @param waitFor is optional expected state (0 = stopped, 1 = running)
 */

IsProcessRunningEx: procedure expose (Globals)
  /* Requires LoadRxUtilEx */
  /* Requires GetPidForProcess */
  /* Requires gAppName */
  /* Sets gPid */
  parse arg waitFor
  do c = 1 to 10
    pid = GetPidForProcess(gAppName)
    signal on Error		/* Restore GetPidForProcess override */
    running = pid \== ''
    if waitFor == '' | waitFor == running then leave
    call SysSleep 1
  end
  gPid = pid
  if gVerbose & running then do
    info = Sys2QueryProcess(pid, 'P')
    say 'pid         ppid ptype pri  cpu time executable name'
    parse var info p pp pt tail
    say left(p, 4) left('('d2x(pid)')', 6) left(pp, 4) left(pt, 5) tail
  end
  return running

/* end IsProcessRunningEx */

/*=== LoadRxu() Load rxu functions ===*/

LoadRxu:
  /* We must RxFuncQuery RxuTerm because
     RxuTerm does not deregister RxuInit
  */
  if RxFuncQuery('RxuTerm') then do
    /* We must RxFuncQuery RxuInit because it may already be loaded */
    if RxFuncQuery('RxuInit') then do
      call RxFuncAdd 'RxuInit','RXU','RxuInit'
      if RESULT then
	call Die 'Cannot load RXU - RxFuncAdd returned' RESULT
    end
    call RxuInit
  end
  return

/* end LoadRxu */

/*=== LoadRxUtilEx() Load Alex's RxUtilEx functions ===*/

LoadRxUtilEx:
  if RxFuncQuery('Sys2LoadFuncs') then do
    call RxFuncAdd 'Sys2LoadFuncs', 'RXUTILEX', 'Sys2LoadFuncs'
    if RESULT then
      call Die 'Cannot load Sys2LoadFuncs'
    call Sys2LoadFuncs
  end
  return

/* end LoadRxUtilEx */

/*=== MakePath(pathparts,...) Make path name from parts ===*/

MakePath: procedure

  /* All parts optional - code guesses what caller means.
     If last arg begins with dot, it is assumed to be a file extension.
     To avoid this behavior, pass empty arg as last arg.
     Empty args are ignored.
     Automatically converts unix slashes to dos slashes.
     If 1st arg is drive letter, it must have trailing colon.
   */

  argCnt = arg()

  path = ''

  do argNum = 1 to argCnt
    s = arg(argNum)
    s = translate(s, '\', '/')		/* Ensure DOS */
    if s == '' & argNum = argCnt then
      iterate				/* Ignore nul last arg */
    if argNum = 1 then
      path = s
    else do
      lead = left(s, 1)
      tail = right(path, 1)
      if tail == ':' & argNum = 2 then
	path = path || s		/* Append path part to drive spec */
      else if lead == '.' & argNum = argCnt then
	path = path || s		/* Assume extension */
      else if tail == '\' & lead == '\' then
	path = path || substr(s, 2)	/* Drop extra backslash */
      else if path \== '' & tail \== '\' & lead \== '\' then
	path = path || '\' || s		/* Ensure have backslash */
      else
	path = path || s
    end
  end /* for */

  return path

/* end MakePath */

/*=== RunKiller(signal) Run process killer, default signal is kill ===*/

/**
 * Sends signal to named process
 * Sends -kill signal unless overridden
 */

RunKiller: procedure expose (Globals)

  /* Requires IsExeInPath */
  /* Requires gPid */
  /* Requires gKiller */
  /* Sets gKiller if not defined */

  parse arg sig

  if sig == '' then
    sig = '-KILL'

  if symbol('gPid') \== 'VAR' then
    call Die 'gPid not defined'

  if \ datatype(gPid, 'W') | gPid < 2 then
    call Die 'Pid "' || gPid || '" is not a valid pid'

  if symbol('gKiller') \= 'VAR' then do
    do 1
      gKiller = 'apache_kill'
      if IsExeInPath(gKiller) then leave
      gKiller = 'emxkill'
      if IsExeInPath(gKiller) then leave
      call Die 'Can not select killer'
    end
  end
  select
  when gKiller == 'apache_kill' then nop
  when gKiller == 'emxkill' then nop
  otherwise
    /* Convert signal name to number */
    sigTbl = '-HUP 1 -KILL -9 -TERM 15 -USR1 16'
    do i = 1 to words(sigTbl) by 2
      if translate(sig) == word(sigTbl, i) then do
	sig = word(sigTbl, i + 1)
	leave
      end
    end
  end
  cmd = gKiller sig gPid
  say cmd || '('d2x(gPid)')'
  '@' || cmd

  return

/* end RunKiller */

/*=== SetFocus() Set focus to our window ===*/

SetFocus: procedure

  /* Take focus if running 4OS2 or fail silently if cannot match window title */
  signal off Error
  '@if "%_DOS%" == "OS2" activate "%@replace[[,*,%_WINTITLE]"'	/* Take focus if running 4OS2 */
  signal on Error
  return

/* end SetFocus */

/*==========================================================================*/
/*=== SkelRexx standards - Delete unused - Move modified above this mark ===*/
/*==========================================================================*/

/*=== Die([message,...]) Write multi-line message to STDERR and die ===*/

Die:
  call lineout 'STDERR', ''
  do i = 1 to arg()
    call lineout 'STDERR', arg(i)
  end
  call lineout 'STDERR', gCmdName 'aborting at line' SIGL || '.'
  call beep 200, 300
  call SysSleep 2
  exit 254

/* end Die */

/*=== Error() Set gErrCondition; report to STDOUT; trace and exit or return if called ===*/

Error:
  say
  parse source . . cmd
  gErrCondition = condition('C')
  say gErrCondition 'signaled at line' SIGL 'of' cmd || '.'
  if condition('D') \= '' then
    say 'REXX reason =' condition('D') || '.'
  if gErrCondition == 'SYNTAX' & symbol('RC') == 'VAR' then
    say 'REXX error =' RC '-' errortext(RC) || '.'
  else if symbol('RC') == 'VAR' then
    say 'RC =' RC || '.'
  say 'Source =' sourceline(SIGL)

  if condition('I') \== 'CALL' | gErrCondition == 'NOVALUE' | gErrCondition == 'SYNTAX' then do
    trace '?A'
    say 'Enter REXX commands to debug failure.  Press enter to exit script.'
    nop
    if symbol('RC') \== 'VAR' then
      RC = 255
    exit RC
  end

  return

/* end Error */

/*=== Halt() Report HALT condition to STDOUT and exit ===*/

Halt:
  say
  parse source . . cmd
  say condition('C') 'signaled at' cmd 'line' SIGL || '.'
  say 'Source =' sourceline(SIGL)
  say 'Sleeping for 2 seconds...'
  call SysSleep 2
  exit 253

/* end Halt */

/*=== LoadRexxUtil() Load RexxUtil functions ===*/

LoadRexxUtil:
  if RxFuncQuery('SysLoadFuncs') then do
    call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
    if RESULT then
      call Die 'Cannot load SysLoadFuncs.'
    call SysLoadFuncs
  end
  return

/* end LoadRexxUtil */

/*=== ScanArgs(cmdLine) Scan command line ===*/

ScanArgs: procedure expose (Globals)

  /* Calls user exits to process arguments and switches */

  parse arg cmdTail
  cmdTail = strip(cmdTail)

  call ScanArgsInit

  /* Ensure optional settings initialized */
  if symbol('SWCTL') \== 'VAR' then
    swCtl = ''				/* Switches that take args, append ? if optional */
  if symbol('KEEPQUOTED') \== 'VAR' then
    keepQuoted = 0			/* Set to 1 to keep arguments quoted */

  /* Scan */
  curArg = ''				/* Current arg string */
  curSwList = ''			/* Current switch list */
  /* curSwArg = '' */			/* Current switch argument, if needed */
  noMoreSw = 0				/* End of switches */

  do while cmdTail \== '' | curArg \== '' | curSwList \== ''

    /* If arg buffer empty, refill */
    if curArg == '' then do
      qChar = left(cmdTail, 1)		/* Remember quote */
      if \ verify(qChar,'''"', 'M') then
	parse var cmdTail curArg cmdTail	/* Not quoted */
      else do
	/* Arg is quoted */
	curArg = ''
	do forever
	  /* Parse dropping quotes */
	  parse var cmdTail (qChar)quotedPart(qChar) cmdTail
	  curArg = curArg || quotedPart
	  /* Check for escaped quote within quoted string (i.e. "" or '') */
	  if left(cmdTail, 1) \== qChar then do
	    cmdTail = strip(cmdTail)	/* Strip leading whitespace */
	    leave			/* Done with this quoted arg */
	  end
	  curArg = curArg || qChar	/* Append quote */
	  if keepQuoted then
	    curArg = curArg || qChar	/* Append escaped quote */
	  parse var cmdTail (qChar) cmdTail	/* Strip quote */
	end /* do forever */
	if keepQuoted then
	  curArg = qChar || curArg || qChar	/* requote */
      end /* if quoted */
    end /* if curArg empty */

    /* If switch buffer empty, refill */
    if curSwList == '' & \ noMoreSw then do
      if left(curArg, 1) == '-' & curArg \== '-' then do
	if curArg == '--' then
	  noMoreSw = 1
	else
	  curSwList = substr(curArg, 2)	/* Remember switch string */
	curArg = ''			/* Mark empty */
	iterate				/* Refill arg buffer */
      end /* if switch */
    end /* if curSwList empty */

    /* If switch in progress */
    if curSwList \== '' then do
      curSw = left(curSwList, 1)	/* Next switch */
      curSwList = substr(curSwList, 2)	/* Drop from pending */
      /* Check switch allows argument, avoid matching ? */
      if pos(curSw, translate(swCtl,,'?')) \= 0 then do
	if curSwList \== '' then do
	  curSwArg = curSwList		/* Use rest of switch string for switch argument */
	  curSwList = ''
	end
	else if curArg \== '' & left(curArg, 1) \== '-' then do
	  curSwArg = curArg		/* Arg string is switch argument */
	  curArg = ''			/* Mark arg string empty */
	end
	else if pos(curSw'?', swCtl) = 0 then
	  call ScanArgsUsage 'Switch "-' || curSw || '" requires an argument'
	else
	  curSwArg = ''			/* Optional arg omitted */
      end

      call ScanArgsSwitch		/* Passing curSw and curSwArg */
      drop curSwArg			/* Must be used by now */
    end /* if switch */

    /* If arg */
    else if curArg \== '' then do
      noMoreSw = 1
      call ScanArgsArg			/* Passing curArg */
      curArg = ''
    end

  end /* while not done */

  call ScanArgsTerm

  return

/* end ScanArgs */

/*=== SetCmdName() Set gCmdName to short script name ===*/

SetCmdName: procedure expose (Globals)
  parse source . . cmd
  cmd = filespec('N', cmd)		/* Chop path */
  c = lastpos('.', cmd)
  if c > 1 then
    cmd = left(cmd, c - 1)		/* Chop extension */
  gCmdName = translate(cmd, xrange('a', 'z'), xrange('A', 'Z'))	/* Lowercase */
  return

/* end SetCmdName */

/*=== SetTmpDir() Set gTmpDir to %TMP with trailing backslash or empty string ===*/

SetTmpDir: procedure expose (Globals)
  s = value('TMP',,gEnv)
  if s \= '' & right(s, 1) \= ':' & right(s, 1) \== '\' then
    s = s'\'				/* Stuff backslash */
  gTmpDir = s
  return

/* end SetTmpDir */

/* eof */
